/**************************************************************************************
Author:         Dave Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Date:           11/19/2009
Program:        materials_vray
Description:    This script allows the user to affect muliple vray materials
                simultaneously
***************************************************************************************/

try (destroyDialog vray_material_rollout) catch()


struct VRayMaterials
(
    vray_materials = #(),
    vray_material_names = #(),
    vray_numeric_properties = #(),
    vray_boolean_properties = #(),
    vray_color_properties = #(),
    vray_map_properties = #(),
    vray_map_multiplier_properties = #(),
    vray_map_boolean_properties = #(),
    
    /**************************************************************************
    Sorts items by name
    **************************************************************************/
    fn sort_by_name item_one item_two =
    (
        case of
        (
            (item_one.name < item_two.name): -1
            (item_one.name > item_two.name): 1
            default: 0
        ) --end case
    ), --end fn
    
    /**************************************************************************
    Collects a list of all vray materials and their names in the current scene
    **************************************************************************/
    fn collect_vray_materials =
    (
        this.vray_materials = getClassInstances VRayMtl
        
        qsort this.vray_materials this.sort_by_name
        
        this.vray_material_names = for current in this.vray_materials collect current.name
    ), --end fn
    
    /**************************************************************************
    Collects a list of all vray material properties and sorts them
    **************************************************************************/
    fn collect_vray_properties =
    (
        local temp_material = VRayMtl()
        
        --CRITICAL! Delete previous data!
        this.vray_numeric_properties = #()
        this.vray_boolean_properties = #()
        this.vray_color_properties = #()
        this.vray_map_properties = #()
        this.vray_map_multiplier_properties = #()
        this.vray_map_boolean_properties = #()
        
        for current_property in (getPropNames temp_material) do
        (
            if (matchPattern current_property pattern:"texmap_*" ignorecase:true) then
            (
                case (classOf (getProperty temp_material current_property)) of
                (
                    (Integer): append vray_map_multiplier_properties (current_property as string)
                    (Float): append vray_map_multiplier_properties (current_property as string)
                    (BooleanClass): append vray_map_boolean_properties (current_property as string)
                    (UndefinedClass): append vray_map_properties (current_property as string)
                    default: print("WARNING: Unknown property found: " + (current_property as string))
                ) --end case
            ) --end if
            else
            (
                case (classOf (getProperty temp_material current_property)) of
                (
                    (Integer): append vray_numeric_properties (current_property as string)
                    (Float): append vray_numeric_properties (current_property as string)
                    (BooleanClass): append vray_boolean_properties (current_property as string)
                    (Color): append vray_color_properties (current_property as string)
                    default: print("WARNING: Unknown property found: " + (current_property as string))
                ) --end case
            ) --end else
        ) --end for
        
        sort this.vray_numeric_properties
        sort this.vray_boolean_properties
        sort this.vray_color_properties
        sort this.vray_map_properties
        sort this.vray_map_multiplier_properties
        sort this.vray_map_boolean_properties
    ), --end fn
    
    /**************************************************************************
    Sets the material property 'property_name' to the value of 'property_value'
    for the materials defined by 'mode'
    **************************************************************************/
    fn set_material_property mode user_selection property_name property_value =
    (
        case mode of
        (
            (1): for index in user_selection do
                    setProperty this.vray_materials[index] property_name property_value
            (2): for current in selection do
                    if current.material != undefined do
                        if (classOf current.material) == VRayMtl do
                            setProperty current.material property_name property_value
            (3): for current in this.vray_materials do
                    setProperty current property_name property_value
        ) --end case
    ) --end fn
) --end struct


( --local scope
    global vray_material_rollout
    local vray_material_ini = getDir #plugcfg + "\\VRayMaterials.ini"
    local vray_material_position
    
    rollout vray_material_rollout "VRay Materials"
    (
        local vray_mats = VRayMaterials()
        
        /***********************************************************************
        Enables/Disables the UI controls based on argument 'state'
        ***********************************************************************/
        fn enable_controls state =
        (
            vray_material_rollout.numeric_properties_dropdown.enabled = state
            vray_material_rollout.numeric_properties_spinner.enabled = state
            vray_material_rollout.boolean_properties_dropdown.enabled = state
            vray_material_rollout.boolean_true_button.enabled = state
            vray_material_rollout.boolean_false_button.enabled = state
            vray_material_rollout.color_properties_dropdown.enabled = state
            vray_material_rollout.color_properties_picker.enabled = state
            vray_material_rollout.map_properties_dropdown.enabled = state
            vray_material_rollout.map_properties_button.enabled = state
            vray_material_rollout.map_multiplier_dropdown.enabled = state
            vray_material_rollout.map_multiplier_spinner.enabled = state
            vray_material_rollout.map_boolean_dropdown.enabled = state
            vray_material_rollout.map_boolean_true_button.enabled = state
            vray_material_rollout.map_boolean_false_button.enabled = state
        ) --end fn
        
        /***********************************************************************
        Refreshes the material names in the list box
        ***********************************************************************/
        fn refresh_vray_material_list =
        (
            vray_mats.collect_vray_materials()
            
            message = "Total: " + (vray_mats.vray_materials.count as string)
            vray_material_rollout.total_label.text = message
            
            vray_material_rollout.material_listbox.items = vray_mats.vray_material_names
        ) --end fn
        
        /***********************************************************************
        Refreshes the property names in the property dropdowns
        ***********************************************************************/
        fn refresh_vray_property_dropdowns =
        (
            vray_mats.collect_vray_properties()
            
            vray_material_rollout.numeric_properties_dropdown.items = vray_mats.vray_numeric_properties
            vray_material_rollout.boolean_properties_dropdown.items = vray_mats.vray_boolean_properties
            vray_material_rollout.color_properties_dropdown.items = vray_mats.vray_color_properties
            vray_material_rollout.map_properties_dropdown.items = vray_mats.vray_map_properties
            vray_material_rollout.map_multiplier_dropdown.items = vray_mats.vray_map_multiplier_properties
            vray_material_rollout.map_boolean_dropdown.items = vray_mats.vray_map_boolean_properties
        ) --end fn
        
        /***********************************************************************
        Returns 'OK' if the user selection and mode are valid. Error message
        otherwise
        ***********************************************************************/
        fn get_validation =
        (
            local user_selection = vray_material_rollout.material_listbox.selection as array
            local validation = "OK"
            
            case vray_material_rollout.mode_radio.state of
            (
                (1): if user_selection.count == 0 do validation = "ERROR: No materials selected"
                (2): if selection.count == 0 do validation = "ERROR: No objects selected"
                (3): if vray_mats.vray_materials.count == 0 do validation = "ERROR: No VRay materials found"
            ) --end case
            
            validation --return value
        ) --end fn
        
        /***********************************************************************
        UI creation
        ***********************************************************************/
        label total_label "Total: 0" pos:[5,5] width:100 height:15
        button update_button "Update" pos:[245,3] width:50 height:15
        multiListBox material_listbox "" pos:[5,20] width:290 height:17
        
        groupBox mode_group "Mode" pos:[5,250] width:290 height:55
            radioButtons mode_radio "" pos:[15,270] width:174 height:32 labels:#("List Select", "Scene Select", "All Materials") columns:2 default:1
            
        groupBox property_group "Properties" pos:[5,310] width:290 height:200
            dropDownList numeric_properties_dropdown "" pos:[15,330] width:200 height:21
            spinner numeric_properties_spinner "" pos:[225,332] width:60 height:15 range:[0,99999,1] type:#float
            dropDownList boolean_properties_dropdown "" pos:[15,360] width:200 height:21
            button boolean_true_button "ON" pos:[225,360] width:30 height:20
            button boolean_false_button "OFF" pos:[255,360] width:30 height:20
            dropDownList color_properties_dropdown "" pos:[15,390] width:200 height:21
            colorPicker color_properties_picker "" pos:[220,390] width:65 height:20
            dropDownList map_properties_dropdown "" pos:[15,420] width:200 height:21
            mapButton map_properties_button "M" pos:[225,420] width:60 height:20
            dropDownList map_multiplier_dropdown "" pos:[15,450] width:200 height:21
            spinner map_multiplier_spinner "" pos:[225,450] width:60 height:15 range:[0,99999,1] type:#float
            dropDownList map_boolean_dropdown "" pos:[15,480] width:200 height:21
            button map_boolean_true_button "ON" pos:[225,480] width:30 height:20
            button map_boolean_false_button "OFF" pos:[255,480] width:30 height:20
        
        label status_label "INFO: Standing by..." pos:[5,520] width:290 height:15
        
        /***********************************************************************
        Rollout open event handler. Prepares the rollout for first time open
        ***********************************************************************/
        on vray_material_rollout open do
        (
            if (classOf VRayMtl) != material do
            (
                enable_controls false
                vray_material_rollout.status_label.text = "ERROR: VRay material not loaded. Check plugin status"
                return false
            ) --end if
            
            refresh_vray_material_list()
            refresh_vray_property_dropdowns()
            enable_controls true
        ) --end on
        
        /***********************************************************************
        Rollout moved event handler. Stores the current position of the rollout
        ***********************************************************************/
        on vray_material_rollout moved pos do
            setIniSetting vray_material_ini "Dialog" "Position" (pos as string)
        
        /***********************************************************************
        Update button clicked event handler. Updates the UI
        ***********************************************************************/
        on update_button pressed do
        (
            if (classOf VRayMtl) != material do
            (
                enable_controls false
                vray_material_rollout.status_label.text = "ERROR: VRay material not loaded. Check plugin status"
                return false
            ) --end if
            
            refresh_vray_material_list()
            refresh_vray_property_dropdowns()
            enable_controls true
            
            status_label.text = "INFO: VRay material list updated"
        ) --end on
        
        /***********************************************************************
        Numeric spinner changed event handler. Applies the new numeric value to
        the selected property of the vray materials
        ***********************************************************************/
        on numeric_properties_spinner changed the_value do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                numeric_properties_dropdown.selected \
                                                the_value
                message = ("INFO: " + numeric_properties_dropdown.selected)
                message += (" set to " + (the_value as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Boolean ON clicked event handler. Turns on the selected property of the
        vray materials
        ***********************************************************************/
        on boolean_true_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                boolean_properties_dropdown.selected \
                                                true
                message = ("INFO: " + boolean_properties_dropdown.selected)
                message += " set to ON"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Boolean OFF clicked event handler. Turns off the selected property of the
        vray materials
        ***********************************************************************/
        on boolean_false_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                boolean_properties_dropdown.selected \
                                                false
                message = ("INFO: " + boolean_properties_dropdown.selected)
                message += " set to OFF"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Color picker clicked event handler. Changes the color of the selected
        property of the vray materials
        ***********************************************************************/
        on color_properties_picker changed the_color do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                color_properties_dropdown.selected \
                                                the_color
                message = ("INFO: " + color_properties_dropdown.selected)
                message += (" set to " + (the_color as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map picker clicked event handler. Assigns the map to the selected
        property of the vray materials
        ***********************************************************************/
        on map_properties_button picked the_map do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_properties_dropdown.selected \
                                                the_map
                message = ("INFO: " + map_properties_dropdown.selected)
                message += (" set to " + ((classOf the_map) as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map spinner changed event handler. Applies the new numeric value to
        the selected property of the vray materials
        ***********************************************************************/
        on map_multiplier_spinner changed the_value do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_multiplier_dropdown.selected \
                                                the_value
                message = ("INFO: " + map_multiplier_dropdown.selected)
                message += (" set to " + (the_value as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map boolean ON clicked event handler. Turns on the selected property of
        the vray materials
        ***********************************************************************/
        on map_boolean_true_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_boolean_dropdown.selected \
                                                true
                message = ("INFO: " + map_boolean_dropdown.selected)
                message += " set to ON"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map boolean OFF clicked event handler. Turns off the selected property
        of the vray materials
        ***********************************************************************/
        on map_boolean_false_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                vray_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_boolean_dropdown.selected \
                                                false
                message = ("INFO: " + map_boolean_dropdown.selected)
                message += " set to OFF"
            ) --end if
            
            status_label.text = message
        ) --end on
    ) --end rollout
    
    vray_material_position = execute (getIniSetting vray_material_ini "Dialog" "Position")
    if vray_material_position == OK do vray_material_position = [100,100]
    if vray_material_position.x > sysinfo.desktopsize.x - 300 do
        vray_material_position.x = sysinfo.desktopsize.x - 305
    if vray_material_position.y > sysinfo.desktopsize.y - 595 do
        vray_material_position.y = sysinfo.desktopsize.y - 595
    if vray_material_position.x < 0 do vray_material_position.x = 0
    if vray_material_position.y < 0 do vray_material_position.y = 0
    createDialog vray_material_rollout 300 535 vray_material_position.x vray_material_position.y style:#(#style_titlebar, #style_minimizebox, #style_border, #style_sysmenu)
) --end scope